import { Component } from 'react';
import PropTypes from 'prop-types';

var State = (function (Component$$1) {
  function State (props, context) {
    Component$$1.call(this, props, context);
    this.broadcast = context.__statty__.broadcast;
    this.inspect = context.__statty__.inspect;
    this.gs = this.broadcast.getState;
    this.state = props.state ? props.state : this.gs();
    this.update = this.update.bind(this);
    this.setState = this.setState.bind(this);
  }

  if ( Component$$1 ) State.__proto__ = Component$$1;
  State.prototype = Object.create( Component$$1 && Component$$1.prototype );
  State.prototype.constructor = State;

  State.prototype.update = function update (fn) {
    if (this.props.state) {
      this.setState(fn);
    } else {
      var oldState = this.gs();
      var newState = fn(oldState);
      this.inspect && this.inspect(oldState, newState, fn);
      this.broadcast.setState(newState);
    }
  };

  State.prototype.componentDidMount = function componentDidMount () {
    if (!this.props.state) { this.subscriptionId = this.broadcast.subscribe(this.setState); }
  };

  State.prototype.componentWillUnmount = function componentWillUnmount () {
    this.subscriptionId && this.broadcast.unsubscribe(this.subscriptionId);
  };

  State.prototype.render = function render () {
    var props = this.props;
    if (!props.render) { return null }
    return (
      props.render(
        props.select(props.state ? this.state : this.gs()),
        this.update
      ) || null
    )
  };

  return State;
}(Component));

State.defaultProps = {
  select: function (s) { return s; }
};

State.contextTypes = {
  __statty__: PropTypes.object.isRequired
};

State.propTypes = {
  update: PropTypes.func,
  state: PropTypes.object,
  render: PropTypes.func
};

function createBroadcast$1 (initialState) {
  var listeners = {};
  var id = 1;
  var _state = initialState;

  function getState () {
    return _state
  }

  function setState (state) {
    _state = state;
    var keys = Object.keys(listeners);
    var i = 0;
    var len = keys.length;
    for (; i < len; i++) {
      // if a listener gets unsubscribed during setState we just skip it
      if (listeners[keys[i]]) { listeners[keys[i]](state); }
    }
  }

  // subscribe to changes and return the subscriptionId
  function subscribe (listener) {
    if (typeof listener !== 'function') {
      throw new Error('listener must be a function.')
    }
    var currentId = id;
    listeners[currentId] = listener;
    id += 1;
    return currentId
  }

  // remove subscription by removing the listener function
  function unsubscribe (id) {
    listeners[id] = undefined;
  }

  return { getState: getState, setState: setState, subscribe: subscribe, unsubscribe: unsubscribe }
}

var Provider = (function (Component$$1) {
  function Provider (props, context) {
    Component$$1.call(this, props, context);
    this.broadcast = createBroadcast$1(props.state);
  }

  if ( Component$$1 ) Provider.__proto__ = Component$$1;
  Provider.prototype = Object.create( Component$$1 && Component$$1.prototype );
  Provider.prototype.constructor = Provider;

  Provider.prototype.getChildContext = function getChildContext () {
    return {
      __statty__: {
        broadcast: this.broadcast,
        inspect: this.props.inspect
      }
    }
  };

  Provider.prototype.render = function render () {
    var c = this.props.children;
    return Array.isArray(c) ? c[0] : c
  };

  return Provider;
}(Component));

Provider.childContextTypes = {
  __statty__: PropTypes.object
};

Provider.propTypes = {
  inspect: PropTypes.func,
  state: PropTypes.object
};

export { Provider, State };
